import os
from unittest.mock import Mock
from mode.utils.imports import symbol_by_name
import pytest


@pytest.fixture()
def imp():
    return Mock(name='imp')


def test_when_ValueError(imp):
    imp.side_effect = ValueError
    with pytest.raises(ValueError):
        symbol_by_name('foo.bar:Baz', imp=imp)


@pytest.mark.parametrize('exc', [AttributeError, ImportError])
def test_when_ImportError(exc, imp):
    imp.side_effect = exc()
    with pytest.raises(exc):
        symbol_by_name('foo.bar:Baz', imp=imp, default=None)


@pytest.mark.parametrize('exc', [AttributeError, ImportError])
def test_when_ImportError__with_default(exc, imp):
    imp.side_effect = exc()
    assert symbol_by_name('foo.bar:Baz', imp=imp, default='f') == 'f'


def test_module():
    assert symbol_by_name('os') is os
